#Makefile to create figures for the report

vp-clip= bclip=1500 eclip=4137
vs-clip= bclip=0 eclip=2364
rho-clip= bclip=1000.0 eclip=2486
fatcolor= axisfat=5 labelfat=5 barfat=5
##################
#SEDIMENT TEST
#true model parameters
${R}/true-vp.v: ${D}/vp.sed.H 
	Grey <$< color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Vp [m/s]" dbarnum=800 ${vp-clip} ${FIG}  ${fatcolor}
${R}/true-vs.v: ${D}/vs.sed.H
	Grey <$< color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Vs [m/s]" dbarnum=500 ${vs-clip} ${FIG} ${fatcolor}
${R}/true-rho.v: ${D}/rho.sed.H 
	Grey <$< color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Density [kg/m3]" dbarnum=500 ${rho-clip} ${FIG}	${fatcolor}

${R}/comb-true-model.v: ${R}/true-vp.v ${R}/true-vs.v ${R}/true-rho.v
	vp_SideBySideIso $^ >$@

#starting model parameters
${R}/start-vp.v: ${D}/vp.sed.smooth.H 
	Grey <$< color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Vp [m/s]" dbarnum=800 ${vp-clip} ${FIG} ${fatcolor}
${R}/start-vs.v: ${D}/vs.sed.smooth.H 
	Grey <$< color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Vs [m/s]" dbarnum=500 ${vs-clip} ${FIG} ${fatcolor}
${R}/start-rho.v: ${D}/rho.sed.smooth.H 
	Grey <$< color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Density [kg/m3]" dbarnum=500 ${rho-clip} ${FIG}	${fatcolor}
	
${R}/comb-start-model.v: ${R}/start-vp.v ${R}/start-vs.v ${R}/start-rho.v
	vp_SideBySideIso $^ >$@

#Recorded data shot-gather
${R}/true-pressure.v: pressure_vels_sed.H
	Window3d < pressure_vels_sed.H n2=1 n4=1 max1=4 | Grey pclip=95 o2=0 d2=10 label2="Offset [m]" ${FIG} ${fatcolor}
${R}/true-vx.v: pressure_vels_sed.H
	Window3d < pressure_vels_sed.H n2=1 f2=1 n4=1 max1=4 | Grey pclip=95 o2=0 d2=10 label2="Offset [m]" ${FIG} ${fatcolor}
${R}/true-vz.v: pressure_vels_sed.H
	Window3d < pressure_vels_sed.H n2=1 f2=2 n4=1 max1=4 | Grey pclip=95 o2=0 d2=10 label2="Offset [m]" ${FIG} ${fatcolor}	

${R}/comb-true-data.v: ${R}/true-pressure.v ${R}/true-vx.v ${R}/true-vz.v
	vp_SideBySideIso $^ >$@
	
	
#Estimation of the Hessian
${R}/hessian-sed%.v: elastic_hessian_sed%.H
	Grey < elastic_hessian_sed$*.H grid=y o1=0 g1num=3000 g2num=5000 gridfat=4 title=" " n1tic=0 n2tic=0 wantaxis=0  out=tmp.v >/dev/null
	vp_annotate < tmp.v batch=y text=${P}/anno.txt >$@
	rm tmp.v

${R}/comb-hess.v: ${R}/hessian-sed5.v ${R}/hessian-sed10.v ${R}/hessian-sed15.v
	vp_SideBySideIso $^ >$@
	

#Objective function comparison
${R}/obj-freq%.v: sed_inversion_bp%_diag.H sed_inversion_bp%_diag.H 
	Cat junk.H sed_inversion_bp$*_diag/obj_sed_inversion_bp$*_diag.H axis=2 | Scale | Graph curvelabel="CG:diagonal GN" label2="Relative objective function" legend=y plotfat=5 axisfat=5 legendfat=5 labelfat=5 legendloc=tr plotcol=5,6  ${FIG}
	
${R}/comb-obj.v: ${R}/obj-freq5.v ${R}/obj-freq10.v ${R}/obj-freq15.v ${R}/obj-freq20.v
	vppen < ${R}/obj-freq5.v vpstyle=n xshift=0 yshift=0 > 1.v
	vppen < ${R}/obj-freq10.v vpstyle=n xshift=11.5 yshift=0 > 2.v
	vppen < ${R}/obj-freq15.v vpstyle=n xshift=23.0 yshift=0 > 3.v
	vppen < ${R}/obj-freq20.v vpstyle=n xshift=34.5 yshift=0 > 4.v
	vppen 1.v 2.v 3.v 4.v vpstyle=n erase=n scale=0.25 >$@
	rm 1.v 2.v 3.v 4.v
	
${R}/comb-obj-rep.v: ${R}/obj-freq5.v ${R}/obj-freq10.v ${R}/obj-freq15.v ${R}/obj-freq20.v
	vp_SideBySideIso ${R}/obj-freq5.v ${R}/obj-freq10.v | vppen xshift=0 yshift=5 >1.v
	vp_SideBySideIso ${R}/obj-freq15.v ${R}/obj-freq20.v| vppen vpstyle=n xshift=0  yshift=0  >2.v
	vppen 1.v 2.v vpstyle=n erase=n >$@
	rm 1.v 2.v
	

#Inverted models
#CG
${R}/inv-CG-vp.v: sed_inversion_bp20.H
	Window3d < sed_inversion_bp20/inverted_sed_inversion_bp20.H  min1=0 min2=0 n1=350 n2=500 n3=1 | Grey color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Vp [m/s]" dbarnum=800 ${vp-clip} ${FIG} ${fatcolor}

${R}/inv-CG-vs.v: sed_inversion_bp20.H 
	Window3d < sed_inversion_bp20/inverted_sed_inversion_bp20.H  min1=0 min2=0 n1=350 n2=500 n3=1 f3=1 | Grey color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Vs [m/s]" dbarnum=800 ${vs-clip} ${FIG} ${fatcolor}

${R}/inv-CG-rho.v: sed_inversion_bp20.H 
	Window3d < sed_inversion_bp20/inverted_sed_inversion_bp20.H  min1=0 min2=0 n1=350 n2=500 n3=1 f3=2 | Grey color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Density [kg/m3]" dbarnum=800 ${rho-clip} ${FIG} ${fatcolor}

${R}/comb-CG-model.v: ${R}/inv-CG-vp.v ${R}/inv-CG-vs.v ${R}/inv-CG-rho.v
	vp_SideBySideIso $^ >$@	
	
#approximated GN
${R}/inv-GN-vp.v: sed_inversion_bp20_diag.H 
	Window3d < sed_inversion_bp20_diag/inverted_sed_inversion_bp20_diag.H  min1=0 min2=0 n1=350 n2=500 n3=1 | Grey color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Vp [m/s]" dbarnum=800 ${vp-clip} ${FIG} ${fatcolor}

${R}/inv-GN-vs.v: sed_inversion_bp20_diag.H 
	Window3d < sed_inversion_bp20_diag/inverted_sed_inversion_bp20_diag.H  min1=0 min2=0 n1=350 n2=500 n3=1 f3=1 | Grey color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Vs [m/s]" dbarnum=800 ${vs-clip} ${FIG} ${fatcolor}

${R}/inv-GN-rho.v: sed_inversion_bp20_diag.H 
	Window3d < sed_inversion_bp20_diag/inverted_sed_inversion_bp20_diag.H  min1=0 min2=0 n1=350 n2=500 n3=1 f3=2 | Grey color=j newclip=1 wantscalebar=1 label1="x [m]" label2="z [m]" label3="Density [kg/m3]" dbarnum=800 ${rho-clip} ${FIG} ${fatcolor}

${R}/comb-GN-model.v: ${R}/inv-GN-vp.v ${R}/inv-GN-vs.v ${R}/inv-GN-rho.v
	vp_SideBySideIso $^ >$@	



