#Makefile to compute approximated inverse hessian 

#SINGLE REFLECTOR
#####	
#Estimating Gauss-Newton Hessian
#Generating spikes
pert_spikes_vp.H pert_spikes_vs.H pert_spikes_rho.H:
	Spike ${model_par} nsp=5 k1=10,60,110,160,210 k2=1,1,1,1,1 mag=1.0 > tmp_spike.H
	Window3d < tmp_spike.H n2=100 >tmp_spike1.H
	Cat tmp_spike1.H tmp_spike1.H tmp_spike1.H tmp_spike1.H tmp_spike1.H tmp_spike1.H tmp_spike1.H tmp_spike1.H axis=2 >tmp_spike2.H
	Pad <tmp_spike2.H n3=3 >pert_spikes_vp.H
	Pad <tmp_spike2.H beg3=1 end3=1 >pert_spikes_vs.H
	Pad <tmp_spike2.H beg3=2 >pert_spikes_rho.H
	Rm tmp_spike*.H
	

#Born test by projecting spikes in model space
spikes_%_data_space.H: solver_prog ${B}/Bornfast_elastic_fw.x ${D}/lame_sin_smooth.H ${D}/mu_sin_smooth.H ${D}/rho_sin_smooth.H ${D}/wavelet_elastic_AVO.H ${P}/source_unif_AVO.p ${P}/rec_unif_AVO.p pert_spikes_%.H data_mask_sin.H ${D}/vp_sin_smooth.H ${D}/vs_sin_smooth.H ${D}/rho_sin_smooth.H
	mkdir -p wrk
	cp ${D}/lame_sin_smooth.H ${D}/mu_sin_smooth.H ${D}/rho_sin_smooth.H ${D}/wavelet_elastic_AVO.H wrk
	#Conversion to lame/mu/rho
	./bin/dvpdvsdrho2dmodrho.py vp0=./Dat/vp_sin_smooth.H vs0=./Dat/vs_sin_smooth.H rho0=./Dat/rho_sin_smooth.H pert_in=pert_spikes_$*.H pert_out=wrk/pert_tmp_spike$*.H adj=0
	#Born application
	${B}/bte_elastic_born_fw.py pbs_template=${PBS_TMPL} queues=${SEPQ},default queues_cap=${queue_cap} path_out=wrk prefix=spike_data_$* nshots=${nshots} nshots_perjob=1 lambda=lame_sin_smooth.H mu=mu_sin_smooth.H rho=rho_sin_smooth.H par_rec=../${P}/rec_unif_AVO.p par_sou=../Par/source_unif_AVO.p wavelet=wavelet_elastic_AVO.H dt=8.0000004E-04 pert=pert_tmp_spike$*.H padded=0 total_jobs_cap=${total_jobs}
	Cat3d `ls wrk/spike_data_$*-*.H | sort -V` axis=4 > $@
	Solver_ops file1=$@ file2=data_mask_sin.H op=multiply
	
#Born adj on spike data	
spikes_%_model_space.H: solver_prog ${B}/Bornfast_elastic_adj.x ${D}/lame_sin_smooth.H ${D}/mu_sin_smooth.H ${D}/rho_sin_smooth.H ${D}/wavelet_elastic_AVO.H ${P}/rec_unif_AVO.p ${P}/source_unif_AVO.p spikes_%_data_space.H ${D}/vp_sin_smooth.H ${D}/vs_sin_smooth.H ${D}/rho_sin_smooth.H
	mkdir -p wrk
	cp ${D}/lame_sin_smooth.H ${D}/mu_sin_smooth.H ${D}/rho_sin_smooth.H ${D}/wavelet_elastic_AVO.H spikes_$*_data_space.H wrk
	${B}/bte_elastic_born_adj.py pbs_template=${PBS_TMPL} queues=${SEPQ},default queues_cap=${queue_cap} path_out=wrk prefix=spike_model_$* nshots=${nshots} nshots_perjob=1 lambda=lame_sin_smooth.H mu=mu_sin_smooth.H rho=rho_sin_smooth.H par_rec=../${P}/rec_unif_AVO.p par_sou=../${P}/source_unif_AVO.p wavelet=wavelet_elastic_AVO.H dt=8.0000004E-04 data=spikes_$*_data_space.H total_jobs_cap=${total_jobs}
	Cat3d `ls wrk/spike_model_$*-*.H | sort -V` axis=4 > tmp_application_$@	
	Transp <tmp_application_$@ plane=24 | Stack3d | Transp plane=23 | Window3d min1=0 min2=0 n1=250 n2=800 > total_tmp_application_$@	
	#Conversion to vp/vs/rho
	./bin/dvpdvsdrho2dmodrho.py vp0=./Dat/vp_sin_smooth.H vs0=./Dat/vs_sin_smooth.H rho0=./Dat/rho_sin_smooth.H pert_in=total_tmp_application_$@ pert_out=$@ adj=1
	Rm tmp_application_$@ total_tmp_application_$@ 
	
#Estimating Hessian elements from spike applications
diag_%.H: spikes_%_model_space.H
	Window3d <$< f1=9 j1=50 j2=100 > pippo.H
	Interp < pippo.H type=1 o1out=0.0 o2out=0.0 d1out=7.0 d2out=7.0 n1out=250 n2out=800 | Pad beg1=120 beg2=120 end1=120 end2=120 extend=1 > $@
	Rm pippo.H

elastic_hessian_sin.H: spikes_vp_model_space.H spikes_vs_model_space.H spikes_rho_model_space.H
	Reshape < spikes_vp_model_space.H reshape=1,3 > tmp_hess1.H
	Reshape < spikes_vs_model_space.H reshape=1,3 > tmp_hess2.H
	Reshape < spikes_rho_model_space.H reshape=1,3 > tmp_hess3.H
	Cat tmp_hess1.H tmp_hess2.H tmp_hess3.H axis=1 >$@
	Rm tmp_hess*.H

#############################################################

#separating the various components of the Hessian matrix
${D}/hess_11.H ${D}/hess_12.H ${D}/hess_13.H: diag_vp.H
	Window3d < $< n3=1 >${D}/hess_11.H
	Window3d < $< n3=1 f3=1 >${D}/hess_12.H
	Window3d < $< n3=1 f3=2 >${D}/hess_13.H
	
${D}/hess_22.H ${D}/hess_23.H: diag_vs.H
	Window3d < $< n3=1 f3=1 >${D}/hess_22.H
	Window3d < $< n3=1 f3=2 >${D}/hess_23.H
	
${D}/hess_33.H: diag_rho.H
	Window3d < $< n3=1 f3=2 >${D}/hess_33.H

###	
#Diagonal inverse only
${D}/diag_hess_inv.H ${D}/norm_diag_hess_inv.H: ${D}/hess_11.H ${D}/hess_22.H ${D}/hess_33.H
	Cat $^ axis=3 > junk_hess.H
	Math file1=junk_hess.H exp='1.0/file1'>${D}/diag_hess_inv.H
	Math file1=junk_hess.H exp='1.0/file1' | Scale >${D}/norm_diag_hess_inv.H
	Rm junk_hess.H


#Scaling to avoid numerical truncation on very small numbers
scale_hess_det=1e39
scale_hess_2x2=1e26
scale_hess_inv=1e13
###	exp="file1*(file4*file6-file5*file5)-file2*(file2*file6-file5*file3)+file3*(file2*file5-file4*file3)"
#Tridiagonal block matrix inverse 
${D}/tri_hess_inv_1col.H ${D}/tri_hess_inv_2col.H ${D}/tri_hess_inv_3col.H ${D}/determ_tri_hess.H: ${D}/hess_11.H ${D}/hess_12.H ${D}/hess_13.H ${D}/hess_22.H ${D}/hess_23.H ${D}/hess_33.H
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H exp="file1*${scale_hess_det}*(file4*file6-file5*file5)-file2*${scale_hess_det}*(file2*file6-file5*file3)+file3*${scale_hess_det}*(file2*file5-file4*file3)" > ${D}/determ_tri_hess.H
	#First column
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H file7=${D}/determ_tri_hess.H exp="(file4*file6-file5*file5)*${scale_hess_2x2}/file7" > tmp_tri_hess_11.H
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H file7=${D}/determ_tri_hess.H exp="(file5*file3-file2*file6)*${scale_hess_2x2}/file7" > tmp_tri_hess_21.H
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H file7=${D}/determ_tri_hess.H exp="(file2*file5-file4*file3)*${scale_hess_2x2}/file7" > tmp_tri_hess_31.H
	Cat tmp_tri_hess_11.H tmp_tri_hess_21.H tmp_tri_hess_31.H >${D}/tri_hess_inv_1col.H
	Solver_ops file1=${D}/tri_hess_inv_1col.H scale1_r=${scale_hess_inv} op=scale
	#Second column
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H file7=${D}/determ_tri_hess.H exp="(file3*file5-file2*file6)*${scale_hess_2x2}/file7" > tmp_tri_hess_12.H
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H file7=${D}/determ_tri_hess.H exp="(file1*file6-file3*file3)*${scale_hess_2x2}/file7" > tmp_tri_hess_22.H
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H file7=${D}/determ_tri_hess.H exp="(file2*file3-file1*file5)*${scale_hess_2x2}/file7" > tmp_tri_hess_32.H
	Cat tmp_tri_hess_12.H tmp_tri_hess_22.H tmp_tri_hess_32.H >${D}/tri_hess_inv_2col.H
	Solver_ops file1=${D}/tri_hess_inv_2col.H scale1_r=${scale_hess_inv} op=scale
	#Third column
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H file7=${D}/determ_tri_hess.H exp="(file2*file5-file3*file4)*${scale_hess_2x2}/file7" > tmp_tri_hess_13.H
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H file7=${D}/determ_tri_hess.H exp="(file3*file2-file1*file5)*${scale_hess_2x2}/file7" > tmp_tri_hess_23.H
	Math file1=${D}/hess_11.H file2=${D}/hess_12.H file3=${D}/hess_13.H file4=${D}/hess_22.H file5=${D}/hess_23.H file6=${D}/hess_33.H file7=${D}/determ_tri_hess.H exp="(file1*file4-file2*file2)*${scale_hess_2x2}/file7" > tmp_tri_hess_33.H
	Cat tmp_tri_hess_13.H tmp_tri_hess_23.H tmp_tri_hess_33.H >${D}/tri_hess_inv_3col.H
	Solver_ops file1=${D}/tri_hess_inv_3col.H scale1_r=${scale_hess_inv} op=scale
	Rm tmp_tri_hess_*.H
	

${D}/norm_tri_hess_inv_1col.H ${D}/norm_tri_hess_inv_2col.H ${D}/norm_tri_hess_inv_3col.H: ${D}/tri_hess_inv_1col.H ${D}/tri_hess_inv_2col.H ${D}/tri_hess_inv_3col.H
	Cat3d $^ axis=4 | Scale > junk.H
	Window3d <junk.H n4=1 >${D}/norm_tri_hess_inv_1col.H
	Window3d <junk.H n4=1 f4=1 >${D}/norm_tri_hess_inv_2col.H
	Window3d <junk.H n4=1 f4=2 >${D}/norm_tri_hess_inv_3col.H
	Rm junk.H

###
#Testing tri-diagonal inverse
test_inv_col_1.H test_inv_col_2.H test_inv_col_3.H: ${D}/tri_hess_inv_1col.H ${D}/tri_hess_inv_2col.H ${D}/tri_hess_inv_3col.H ${D}/hess_11.H ${D}/hess_12.H ${D}/hess_13.H
	#First column of I
	Pad n3=3 extend=1 <${D}/hess_11.H > tmp_hess_11.H
	Pad n3=3 extend=1 <${D}/hess_12.H > tmp_hess_12.H
	Pad n3=3 extend=1 <${D}/hess_13.H > tmp_hess_13.H
	Math file1=${D}/tri_hess_inv_1col.H file2=tmp_hess_11.H file3=${D}/tri_hess_inv_2col.H file4=tmp_hess_12.H file5=${D}/tri_hess_inv_3col.H file6=tmp_hess_13.H exp='file1*file2+file3*file4+file5*file6' > test_inv_col_1.H
	#Second column of I
	Pad n3=3 extend=1 <${D}/hess_12.H > tmp_hess_12.H
	Pad n3=3 extend=1 <${D}/hess_22.H > tmp_hess_22.H
	Pad n3=3 extend=1 <${D}/hess_23.H > tmp_hess_23.H
	Math file1=${D}/tri_hess_inv_1col.H file2=tmp_hess_12.H file3=${D}/tri_hess_inv_2col.H file4=tmp_hess_22.H file5=${D}/tri_hess_inv_3col.H file6=tmp_hess_23.H exp='file1*file2+file3*file4+file5*file6' > test_inv_col_2.H
	#Third column of I
	Pad n3=3 extend=1 <${D}/hess_13.H > tmp_hess_13.H
	Pad n3=3 extend=1 <${D}/hess_23.H > tmp_hess_23.H
	Pad n3=3 extend=1 <${D}/hess_33.H > tmp_hess_33.H
	Math file1=${D}/tri_hess_inv_1col.H file2=tmp_hess_13.H file3=${D}/tri_hess_inv_2col.H file4=tmp_hess_23.H file5=${D}/tri_hess_inv_3col.H file6=tmp_hess_33.H exp='file1*file2+file3*file4+file5*file6' > test_inv_col_3.H
	Rm tmp_hess_*.H



test_inverse_tridiag_hessian.H: test_inv_col_1.H test_inv_col_2.H test_inv_col_3.H
	Reshape < test_inv_col_1.H reshape=1,3 > tmp_hess1.H
	Reshape < test_inv_col_2.H reshape=1,3 > tmp_hess2.H
	Reshape < test_inv_col_3.H reshape=1,3 > tmp_hess3.H
	Cat tmp_hess1.H tmp_hess2.H tmp_hess3.H axis=1 >$@
	Rm tmp_hess*.H test_inv_col_1.H test_inv_col_2.H test_inv_col_3.H

#########################################################################################################
#SEDIMENT MODEL
#####	
#Estimating Gauss-Newton Hessian
#Generating spikes
pert_spikes_vp_sed.H pert_spikes_vs_sed.H pert_spikes_rho_sed.H:
	Spike n1=350 o1=0 d1=10 d2=10 o2=0 d3=10 o3=0 nsp=3 k1=110,210,310 k2=1,1,1 mag=1.0 | Pad end2=99 > tmp_spike.H
	Add tmp_spike.H scale=0.0 > null_spike.H
	Cat null_spike.H tmp_spike.H tmp_spike.H tmp_spike.H tmp_spike.H axis=2 >tmp_spike1.H
	Pad <tmp_spike1.H beg1=120 end1=120 beg2=120 end2=120 n3=3 >pert_spikes_vp_sed.H
	Pad <tmp_spike1.H beg1=120 end1=120 beg2=120 end2=120 beg3=1 end3=1 >pert_spikes_vs_sed.H
	Pad <tmp_spike1.H beg1=120 end1=120 beg2=120 end2=120 beg3=2 >pert_spikes_rho_sed.H
	Rm tmp_spike*.H null_spike.H

#Born fwd on spike data	Vp/Vs/Rho 5Hz
spikes_%_data_space_sed5.H: pert_spikes_%_sed.H initial_model_sed.H ${D}/wavelet_elastic_sed_bp5.H ${P}/rec_unif_sed.p ${P}/source_unif_sed.p solver_prog
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp5.H wrk
	Window3d < initial_model_sed.H n3=1 > wrk/vp_spike_sed.H
	Window3d < initial_model_sed.H n3=1 f3=1 > wrk/vs_spike_sed.H
	Window3d < initial_model_sed.H n3=1 f3=2 > wrk/rho_spike_sed.H
	Math file1=wrk/vp_spike_sed.H file2=wrk/vs_spike_sed.H file3=wrk/rho_spike_sed.H exp="file3*(file1*file1-2.0*file2*file2)/1e9" > wrk/lame_spike_sed.H
	Math file1=wrk/vs_spike_sed.H file2=wrk/rho_spike_sed.H exp="file2*file1*file1/1e9" > wrk/mu_spike_sed.H
	./bin/dvpdvsdrho2dmodrho.py vp0=wrk/vp_spike_sed.H vs0=wrk/vs_spike_sed.H rho0=wrk/rho_spike_sed.H pert_in=$< pert_out=wrk/pert_spike_tmp_sed.H adj=0
	./bin/bte_elastic_born_fw.py pbs_template=${PBS_TMPL} queues=${SEPQ},default queues_cap=${queue_cap} path_out=wrk prefix=spike-sed-born-fwd nshots=50 nshots_perjob=1 lambda=lame_spike_sed.H mu=mu_spike_sed.H rho=rho_spike_sed.H par_rec=../${P}/rec_unif_sed.p par_sou=../${P}/source_unif_sed.p wavelet=wavelet_elastic_sed_bp5.H dt=8.0000004E-04 pert=pert_spike_tmp_sed.H padded=1 total_jobs_cap=${total_jobs} mod_padded=1
	Cat3d `ls wrk/spike-sed-born-fwd-*.H | sort -V` axis=4 > wrk/tmp_data_spike_sed.H
	Window3d < wrk/tmp_data_spike_sed.H n2=2 | Stack3d | Transp reshape=1,3,4 plane=23 | Add scale=0.5 > wrk/pressure_born_fwd_spike.H
	Window3d < wrk/tmp_data_spike_sed.H n2=2 f2=3 > wrk/vels_born_fwd_spike.H
	Cat3d wrk/pressure_born_fwd_spike.H wrk/vels_born_fwd_spike.H axis=2 >$@
	rm -f  wrk/vp_spike_sed.H wrk/vs_spike_sed.H wrk/rho_spike_sed.H wrk/lame_spike_sed.H wrk/mu_spike_sed.H wrk/pert_spike_tmp_sed.H wrk/tmp_data_spike_sed.H wrk/pressure_born_fwd_spike.H wrk/vels_born_fwd_spike.H
	rm -f wrk/spike-sed-born-fwd*
#Born adj on spike data	Vp/Vs/Rho 5Hz
spikes_%_model_space_sed5.H: spikes_%_data_space_sed5.H initial_model_sed.H ${D}/wavelet_elastic_sed_bp5.H solver_prog
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp5.H wrk
	Window3d < initial_model_sed.H n3=1 > wrk/vp_spike_sed.H
	Window3d < initial_model_sed.H n3=1 f3=1 > wrk/vs_spike_sed.H
	Window3d < initial_model_sed.H n3=1 f3=2 > wrk/rho_spike_sed.H
	Math file1=wrk/vp_spike_sed.H file2=wrk/vs_spike_sed.H file3=wrk/rho_spike_sed.H exp="file3*(file1*file1-2.0*file2*file2)/1e9" > wrk/lame_spike_sed.H
	Math file1=wrk/vs_spike_sed.H file2=wrk/rho_spike_sed.H exp="file2*file1*file1/1e9" > wrk/mu_spike_sed.H
	Window3d < $< n2=1 | Transp reshape=1,3,4 plane=23 | Add scale=0.5 > wrk/pressure_born_adj_spike.H
	Window3d < $< f2=1 > wrk/vels_born_adj_spike.H
	Add wrk/pressure_born_adj_spike.H scale=0.0 > wrk/null_born_adj_spike.H
	Cat3d wrk/pressure_born_adj_spike.H wrk/pressure_born_adj_spike.H wrk/null_born_adj_spike.H wrk/vels_born_adj_spike.H axis=2 >wrk/tmp_data_spike_sed.H
	./bin/bte_elastic_born_adj.py pbs_template=${PBS_TMPL} queues=${SEPQ},default queues_cap=${queue_cap} path_out=wrk prefix=spike-sed-born-adj nshots=50 nshots_perjob=1 lambda=lame_spike_sed.H mu=mu_spike_sed.H rho=rho_spike_sed.H par_rec=../${P}/rec_unif_sed.p par_sou=../${P}/source_unif_sed.p wavelet=wavelet_elastic_sed_bp5.H dt=8.0000004E-04 data=tmp_data_spike_sed.H total_jobs_cap=${total_jobs} mod_padded=1
	Cat3d `ls wrk/spike-sed-born-adj-*.H | sort -V` axis=4 > wrk/pert_spike_tmp_sed.H	
	Transp < wrk/pert_spike_tmp_sed.H plane=24 | Stack3d | Transp plane=23 > wrk/total_pert_spike_tmp_sed.H
	./bin/dvpdvsdrho2dmodrho.py vp0=wrk/vp_spike_sed.H vs0=wrk/vs_spike_sed.H rho0=wrk/rho_spike_sed.H pert_in=wrk/total_pert_spike_tmp_sed.H pert_out=tmp_result_spikes_mod.H adj=1
	Window3d < tmp_result_spikes_mod.H min1=0 min2=0 n1=350 n2=500 >$@
	rm -f wrk/vp_spike_sed.H wrk/vs_spike_sed.H wrk/rho_spike_sed.H wrk/lame_spike_sed.H wrk/mu_spike_sed.H wrk/pressure_born_adj_spike.H wrk/vels_born_adj_spike.H wrk/null_born_adj_spike.H wrk/tmp_data_spike_sed.H wrk/pert_spike_tmp_sed.H wrk/total_pert_spike_tmp_sed.H	tmp_result_spikes_mod.H
	rm -f  wrk/spike-sed-born-adj*

####
#Born fwd on spike data	Vp/Vs/Rho frequencies higher than 5Hz for diagonal approximation
spikes_vp_data_space_sed%.H spikes_vs_data_space_sed%.H spikes_rho_data_space_sed%.H: ${D}/wavelet_elastic_sed_bp%.H ${P}/rec_unif_sed.p ${P}/source_unif_sed.p solver_prog
	#make sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H
	make pert_spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_sed.H
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp$*.H wrk
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H n3=1 > wrk/vp_spike_sed.H
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H n3=1 f3=1 > wrk/vs_spike_sed.H
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H n3=1 f3=2 > wrk/rho_spike_sed.H
	Math file1=wrk/vp_spike_sed.H file2=wrk/vs_spike_sed.H file3=wrk/rho_spike_sed.H exp="file3*(file1*file1-2.0*file2*file2)/1e9" > wrk/lame_spike_sed.H
	Math file1=wrk/vs_spike_sed.H file2=wrk/rho_spike_sed.H exp="file2*file1*file1/1e9" > wrk/mu_spike_sed.H
	./bin/dvpdvsdrho2dmodrho.py vp0=wrk/vp_spike_sed.H vs0=wrk/vs_spike_sed.H rho0=wrk/rho_spike_sed.H pert_in=pert_spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_sed.H pert_out=wrk/pert_spike_tmp_sed.H adj=0
	./bin/bte_elastic_born_fw.py pbs_template=${PBS_TMPL} queues=${SEPQ},default queues_cap=${queue_cap} path_out=wrk prefix=spike-sed-born-fwd nshots=50 nshots_perjob=1 lambda=lame_spike_sed.H mu=mu_spike_sed.H rho=rho_spike_sed.H par_rec=../${P}/rec_unif_sed.p par_sou=../${P}/source_unif_sed.p wavelet=wavelet_elastic_sed_bp$*.H dt=8.0000004E-04 pert=pert_spike_tmp_sed.H padded=1 total_jobs_cap=${total_jobs} mod_padded=1
	Cat3d `ls wrk/spike-sed-born-fwd-*.H | sort -V` axis=4 > wrk/tmp_data_spike_sed.H
	Window3d < wrk/tmp_data_spike_sed.H n2=2 | Stack3d | Transp reshape=1,3,4 plane=23 | Add scale=0.5 > wrk/pressure_born_fwd_spike.H
	Window3d < wrk/tmp_data_spike_sed.H n2=2 f2=3 > wrk/vels_born_fwd_spike.H
	Cat3d wrk/pressure_born_fwd_spike.H wrk/vels_born_fwd_spike.H axis=2 >$@
	rm -f  wrk/vp_spike_sed.H wrk/vs_spike_sed.H wrk/rho_spike_sed.H wrk/lame_spike_sed.H wrk/mu_spike_sed.H wrk/pert_spike_tmp_sed.H wrk/tmp_data_spike_sed.H wrk/pressure_born_fwd_spike.H wrk/vels_born_fwd_spike.H
	rm -f wrk/spike-sed-born-fwd*
#Born adj on spike data	Vp/Vs/Rho frequencies higher than 5Hz
spikes_vp_model_space_sed%.H spikes_vs_model_space_sed%.H spikes_rho_model_space_sed%.H: solver_prog ${D}/wavelet_elastic_sed_bp%.H
	#make sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H
	make spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_data_space_sed$*.H
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp$*.H wrk
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H n3=1 > wrk/vp_spike_sed.H
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H n3=1 f3=1 > wrk/vs_spike_sed.H
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H n3=1 f3=2 > wrk/rho_spike_sed.H
	Math file1=wrk/vp_spike_sed.H file2=wrk/vs_spike_sed.H file3=wrk/rho_spike_sed.H exp="file3*(file1*file1-2.0*file2*file2)/1e9" > wrk/lame_spike_sed.H
	Math file1=wrk/vs_spike_sed.H file2=wrk/rho_spike_sed.H exp="file2*file1*file1/1e9" > wrk/mu_spike_sed.H
	Window3d < spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_data_space_sed$*.H n2=1 | Transp reshape=1,3,4 plane=23 | Add scale=0.5 > wrk/pressure_born_adj_spike.H
	Window3d < spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_data_space_sed$*.H f2=1 > wrk/vels_born_adj_spike.H
	Add wrk/pressure_born_adj_spike.H scale=0.0 > wrk/null_born_adj_spike.H
	Cat3d wrk/pressure_born_adj_spike.H wrk/pressure_born_adj_spike.H wrk/null_born_adj_spike.H wrk/vels_born_adj_spike.H axis=2 >wrk/tmp_data_spike_sed.H
	./bin/bte_elastic_born_adj.py pbs_template=${PBS_TMPL} queues=${SEPQ},default queues_cap=${queue_cap} path_out=wrk prefix=spike-sed-born-adj nshots=50 nshots_perjob=1 lambda=lame_spike_sed.H mu=mu_spike_sed.H rho=rho_spike_sed.H par_rec=../${P}/rec_unif_sed.p par_sou=../${P}/source_unif_sed.p wavelet=wavelet_elastic_sed_bp$*.H dt=8.0000004E-04 data=tmp_data_spike_sed.H total_jobs_cap=${total_jobs} mod_padded=1
	Cat3d `ls wrk/spike-sed-born-adj-*.H | sort -V` axis=4 > wrk/pert_spike_tmp_sed.H	
	Transp < wrk/pert_spike_tmp_sed.H plane=24 | Stack3d | Transp plane=23 > wrk/total_pert_spike_tmp_sed.H
	./bin/dvpdvsdrho2dmodrho.py vp0=wrk/vp_spike_sed.H vs0=wrk/vs_spike_sed.H rho0=wrk/rho_spike_sed.H pert_in=wrk/total_pert_spike_tmp_sed.H pert_out=tmp_result_spikes_mod.H adj=1
	Window3d < tmp_result_spikes_mod.H min1=0 min2=0 n1=350 n2=500 >$@
	rm -f wrk/vp_spike_sed.H wrk/vs_spike_sed.H wrk/rho_spike_sed.H wrk/lame_spike_sed.H wrk/mu_spike_sed.H wrk/pressure_born_adj_spike.H wrk/vels_born_adj_spike.H wrk/null_born_adj_spike.H wrk/tmp_data_spike_sed.H wrk/pert_spike_tmp_sed.H wrk/total_pert_spike_tmp_sed.H	tmp_result_spikes_mod.H
	rm -f  wrk/spike-sed-born-adj*

####
#Born fwd on spike data	Vp/Vs/Rho frequencies higher than 5Hz for tridiagonal approximation
spikes_vp_data_space_sed%_tri.H spikes_vs_data_space_sed%_tri.H spikes_rho_data_space_sed%_tri.H: ${D}/wavelet_elastic_sed_bp%.H ${P}/rec_unif_sed.p ${P}/source_unif_sed.p solver_prog
	#make sed_inversion_bp$(shell echo $*-5| bc)_tri/inverted_sed_inversion_bp$(shell echo $*-5| bc)_tri.H
	make pert_spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_sed.H
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp$*.H wrk
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_tri/inverted_sed_inversion_bp$(shell echo $*-5| bc)_tri.H n3=1 > wrk/vp_spike_sed.H
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_tri/inverted_sed_inversion_bp$(shell echo $*-5| bc)_tri.H n3=1 f3=1 > wrk/vs_spike_sed.H
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_tri/inverted_sed_inversion_bp$(shell echo $*-5| bc)_tri.H n3=1 f3=2 > wrk/rho_spike_sed.H
	Math file1=wrk/vp_spike_sed.H file2=wrk/vs_spike_sed.H file3=wrk/rho_spike_sed.H exp="file3*(file1*file1-2.0*file2*file2)/1e9" > wrk/lame_spike_sed.H
	Math file1=wrk/vs_spike_sed.H file2=wrk/rho_spike_sed.H exp="file2*file1*file1/1e9" > wrk/mu_spike_sed.H
	./bin/dvpdvsdrho2dmodrho.py vp0=wrk/vp_spike_sed.H vs0=wrk/vs_spike_sed.H rho0=wrk/rho_spike_sed.H pert_in=pert_spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_sed.H pert_out=wrk/pert_spike_tmp_sed.H adj=0
	./bin/bte_elastic_born_fw.py pbs_template=${PBS_TMPL} queues=${SEPQ},default queues_cap=${queue_cap} path_out=wrk prefix=spike-sed-born-fwd nshots=50 nshots_perjob=1 lambda=lame_spike_sed.H mu=mu_spike_sed.H rho=rho_spike_sed.H par_rec=../${P}/rec_unif_sed.p par_sou=../${P}/source_unif_sed.p wavelet=wavelet_elastic_sed_bp$*.H dt=8.0000004E-04 pert=pert_spike_tmp_sed.H padded=1 total_jobs_cap=${total_jobs} mod_padded=1
	Cat3d `ls wrk/spike-sed-born-fwd-*.H | sort -V` axis=4 > wrk/tmp_data_spike_sed.H
	Window3d < wrk/tmp_data_spike_sed.H n2=2 | Stack3d | Transp reshape=1,3,4 plane=23 | Add scale=0.5 > wrk/pressure_born_fwd_spike.H
	Window3d < wrk/tmp_data_spike_sed.H n2=2 f2=3 > wrk/vels_born_fwd_spike.H
	Cat3d wrk/pressure_born_fwd_spike.H wrk/vels_born_fwd_spike.H axis=2 >$@
	rm -f  wrk/vp_spike_sed.H wrk/vs_spike_sed.H wrk/rho_spike_sed.H wrk/lame_spike_sed.H wrk/mu_spike_sed.H wrk/pert_spike_tmp_sed.H wrk/tmp_data_spike_sed.H wrk/pressure_born_fwd_spike.H wrk/vels_born_fwd_spike.H
	rm -f wrk/spike-sed-born-fwd*
#Born adj on spike data	Vp/Vs/Rho frequencies higher than 5Hz
spikes_vp_model_space_sed%_tri.H spikes_vs_model_space_sed%_tri.H spikes_rho_model_space_sed%_tri.H: solver_prog ${D}/wavelet_elastic_sed_bp%.H
	#make sed_inversion_bp$(shell echo $*-5| bc)_tri/inverted_sed_inversion_bp$(shell echo $*-5| bc)_tri.H
	make spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_data_space_sed$*.H
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp$*.H wrk
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_tri/inverted_sed_inversion_bp$(shell echo $*-5| bc)_tri.H n3=1 > wrk/vp_spike_sed.H
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_tri/inverted_sed_inversion_bp$(shell echo $*-5| bc)_tri.H n3=1 f3=1 > wrk/vs_spike_sed.H
	Window3d < sed_inversion_bp$(shell echo $*-5| bc)_tri/inverted_sed_inversion_bp$(shell echo $*-5| bc)_tri.H n3=1 f3=2 > wrk/rho_spike_sed.H
	Math file1=wrk/vp_spike_sed.H file2=wrk/vs_spike_sed.H file3=wrk/rho_spike_sed.H exp="file3*(file1*file1-2.0*file2*file2)/1e9" > wrk/lame_spike_sed.H
	Math file1=wrk/vs_spike_sed.H file2=wrk/rho_spike_sed.H exp="file2*file1*file1/1e9" > wrk/mu_spike_sed.H
	Window3d < spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_data_space_sed$*.H n2=1 | Transp reshape=1,3,4 plane=23 | Add scale=0.5 > wrk/pressure_born_adj_spike.H
	Window3d < spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_data_space_sed$*.H f2=1 > wrk/vels_born_adj_spike.H
	Add wrk/pressure_born_adj_spike.H scale=0.0 > wrk/null_born_adj_spike.H
	Cat3d wrk/pressure_born_adj_spike.H wrk/pressure_born_adj_spike.H wrk/null_born_adj_spike.H wrk/vels_born_adj_spike.H axis=2 >wrk/tmp_data_spike_sed.H
	./bin/bte_elastic_born_adj.py pbs_template=${PBS_TMPL} queues=${SEPQ},default queues_cap=${queue_cap} path_out=wrk prefix=spike-sed-born-adj nshots=50 nshots_perjob=1 lambda=lame_spike_sed.H mu=mu_spike_sed.H rho=rho_spike_sed.H par_rec=../${P}/rec_unif_sed.p par_sou=../${P}/source_unif_sed.p wavelet=wavelet_elastic_sed_bp$*.H dt=8.0000004E-04 data=tmp_data_spike_sed.H total_jobs_cap=${total_jobs} mod_padded=1
	Cat3d `ls wrk/spike-sed-born-adj-*.H | sort -V` axis=4 > wrk/pert_spike_tmp_sed.H	
	Transp < wrk/pert_spike_tmp_sed.H plane=24 | Stack3d | Transp plane=23 > wrk/total_pert_spike_tmp_sed.H
	./bin/dvpdvsdrho2dmodrho.py vp0=wrk/vp_spike_sed.H vs0=wrk/vs_spike_sed.H rho0=wrk/rho_spike_sed.H pert_in=wrk/total_pert_spike_tmp_sed.H pert_out=tmp_result_spikes_mod.H adj=1
	Window3d < tmp_result_spikes_mod.H min1=0 min2=0 n1=350 n2=500 >$@
	rm -f wrk/vp_spike_sed.H wrk/vs_spike_sed.H wrk/rho_spike_sed.H wrk/lame_spike_sed.H wrk/mu_spike_sed.H wrk/pressure_born_adj_spike.H wrk/vels_born_adj_spike.H wrk/null_born_adj_spike.H wrk/tmp_data_spike_sed.H wrk/pert_spike_tmp_sed.H wrk/total_pert_spike_tmp_sed.H	tmp_result_spikes_mod.H
	rm -f  wrk/spike-sed-born-adj*



#Viewing the result of the applications
elastic_hessian_sed%.H: 
	make spikes_vp_model_space_sed$*.H 
	make spikes_vs_model_space_sed$*.H 
	make spikes_rho_model_space_sed$*.H
	Reshape < spikes_vp_model_space_sed$*.H reshape=1,3 | Window3d f1=50 > tmp_hess1.H
	Reshape < spikes_vs_model_space_sed$*.H reshape=1,3 | Window3d f1=50 > tmp_hess2.H
	Reshape < spikes_rho_model_space_sed$*.H reshape=1,3| Window3d f1=50 > tmp_hess3.H
	Cat tmp_hess1.H tmp_hess2.H tmp_hess3.H axis=1 >$@
	Rm tmp_hess*.H

#Estimating Hessian elements from spike applications
diag_vp_sed%.H diag_vs_sed%.H diag_rho_sed%.H: 
	make spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_model_space_sed$*.H
	Window3d <spikes_$(shell echo $@ | grep -o "vp\|vs\|rho")_model_space_sed$*.H f1=109 j1=100 f2=100 j2=100 > pippo.H
	Interp < pippo.H type=1 o1out=1090 o2out=1000 d1out=10.0 d2out=10.0 n1out=200 n2out=300 | Pad beg1=229 beg2=220 end1=161 end2=220 extend=1 > $@
	Rm pippo.H

################
#separating the various components of the Hessian matrix
${D}/hess_11_sed%.H: diag_vp_sed%.H
	Window3d < $< n3=1 >${D}/hess_11_sed$*.H
	
${D}/hess_12_sed%.H ${D}/hess_22_sed%.H ${D}/hess_23_sed%.H: diag_vs_sed%.H
	Window3d < $< n3=1 		>${D}/hess_12_sed$*.H
	Window3d < $< n3=1 f3=1 >${D}/hess_22_sed$*.H
	Window3d < $< n3=1 f3=2 >${D}/hess_23_sed$*.H
	
${D}/hess_13_sed%.H ${D}/hess_33_sed%.H: diag_rho_sed%.H
	Window3d < $< n3=1	 	>${D}/hess_13_sed$*.H
	Window3d < $< n3=1 f3=2 >${D}/hess_33_sed$*.H
	
###	
#Diagonal inverse only
${D}/diag_hess_inv_sed%.H ${D}/norm_diag_hess_inv_sed%.H: ${D}/hess_11_sed%.H ${D}/hess_22_sed%.H ${D}/hess_33_sed%.H
	Cat $^ axis=3 > junk_hess.H
	Math file1=junk_hess.H exp='1.0/file1'>${D}/diag_hess_inv_sed$*.H
	Math file1=junk_hess.H exp='1.0/file1' | Scale >${D}/norm_diag_hess_inv_sed$*.H
	Rm junk_hess.H	
	
	
#Tridiagonal approximation
#Scaling to avoid numerical truncation on very small numbers
scale_hess_det_sed=1e39
scale_hess_2x2_sed=1e26
scale_hess_inv_sed=1e13
###	exp="file1*(file4*file6-file5*file5)-file2*(file2*file6-file5*file3)+file3*(file2*file5-file4*file3)"
#Tridiagonal block matrix inverse 
${D}/tri_hess_inv_1col_sed5.H ${D}/tri_hess_inv_2col_sed5.H ${D}/tri_hess_inv_3col_sed5.H ${D}/determ_tri_hess_sed5.H: ${D}/hess_11_sed5.H ${D}/hess_12_sed5.H ${D}/hess_13_sed5.H ${D}/hess_22_sed5.H ${D}/hess_23_sed5.H ${D}/hess_33_sed5.H
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H exp="file1*${scale_hess_det_sed}*(file4*file6-file5*file5)-file2*${scale_hess_det_sed}*(file2*file6-file5*file3)+file3*${scale_hess_det_sed}*(file2*file5-file4*file3)" > ${D}/determ_tri_hess_sed5.H
	#First column
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H file7=${D}/determ_tri_hess_sed5.H exp="(file4*file6-file5*file5)*${scale_hess_2x2_sed}/file7" > tmp_tri_hess_11_sed5.H
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H file7=${D}/determ_tri_hess_sed5.H exp="(file5*file3-file2*file6)*${scale_hess_2x2_sed}/file7" > tmp_tri_hess_21_sed5.H
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H file7=${D}/determ_tri_hess_sed5.H exp="(file2*file5-file4*file3)*${scale_hess_2x2_sed}/file7" > tmp_tri_hess_31_sed5.H
	Cat tmp_tri_hess_11_sed5.H tmp_tri_hess_21_sed5.H tmp_tri_hess_31_sed5.H >${D}/tri_hess_inv_1col_sed5.H
	Solver_ops file1=${D}/tri_hess_inv_1col_sed5.H scale1_r=${scale_hess_inv_sed} op=scale
	#Second column
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H file7=${D}/determ_tri_hess_sed5.H exp="(file3*file5-file2*file6)*${scale_hess_2x2_sed}/file7" > tmp_tri_hess_12_sed5.H
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H file7=${D}/determ_tri_hess_sed5.H exp="(file1*file6-file3*file3)*${scale_hess_2x2_sed}/file7" > tmp_tri_hess_22_sed5.H
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H file7=${D}/determ_tri_hess_sed5.H exp="(file2*file3-file1*file5)*${scale_hess_2x2_sed}/file7" > tmp_tri_hess_32_sed5.H
	Cat tmp_tri_hess_12_sed5.H tmp_tri_hess_22_sed5.H tmp_tri_hess_32_sed5.H >${D}/tri_hess_inv_2col_sed5.H
	Solver_ops file1=${D}/tri_hess_inv_2col_sed5.H scale1_r=${scale_hess_inv_sed} op=scale
	#Third column
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H file7=${D}/determ_tri_hess_sed5.H exp="(file2*file5-file3*file4)*${scale_hess_2x2_sed}/file7" > tmp_tri_hess_13_sed5.H
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H file7=${D}/determ_tri_hess_sed5.H exp="(file3*file2-file1*file5)*${scale_hess_2x2_sed}/file7" > tmp_tri_hess_23_sed5.H
	Math file1=${D}/hess_11_sed5.H file2=${D}/hess_12_sed5.H file3=${D}/hess_13_sed5.H file4=${D}/hess_22_sed5.H file5=${D}/hess_23_sed5.H file6=${D}/hess_33_sed5.H file7=${D}/determ_tri_hess_sed5.H exp="(file1*file4-file2*file2)*${scale_hess_2x2_sed}/file7" > tmp_tri_hess_33_sed5.H
	Cat tmp_tri_hess_13_sed5.H tmp_tri_hess_23_sed5.H tmp_tri_hess_33_sed5.H >${D}/tri_hess_inv_3col_sed5.H
	Solver_ops file1=${D}/tri_hess_inv_3col_sed5.H scale1_r=${scale_hess_inv_sed} op=scale
	Rm tmp_tri_hess_*_sed5.H
	
${D}/norm_tri_hess_inv_1col_sed5.H ${D}/norm_tri_hess_inv_2col_sed5.H ${D}/norm_tri_hess_inv_3col_sed5.H: ${D}/tri_hess_inv_1col_sed5.H ${D}/tri_hess_inv_2col_sed5.H ${D}/tri_hess_inv_3col_sed5.H
	Cat3d $^ axis=4 | Scale > junk.H
	Window3d <junk.H n4=1 >${D}/norm_tri_hess_inv_1col_sed5.H
	Window3d <junk.H n4=1 f4=1 >${D}/norm_tri_hess_inv_2col_sed5.H
	Window3d <junk.H n4=1 f4=2 >${D}/norm_tri_hess_inv_3col_sed5.H
	Rm junk.H	
	
###
#Testing tri-diagonal inverse
test_inv_col_1_sed5.H test_inv_col_2_sed5.H test_inv_col_3_sed5.H: ${D}/tri_hess_inv_1col_sed5.H ${D}/tri_hess_inv_2col_sed5.H ${D}/tri_hess_inv_3col_sed5.H ${D}/hess_11.H ${D}/hess_12_sed5.H ${D}/hess_13_sed5.H
	#First column of I
	Pad n3=3 extend=1 <${D}/hess_11_sed5.H > tmp_hess_11_sed5.H
	Pad n3=3 extend=1 <${D}/hess_12_sed5.H > tmp_hess_12_sed5.H
	Pad n3=3 extend=1 <${D}/hess_13_sed5.H > tmp_hess_13_sed5.H
	Math file1=${D}/tri_hess_inv_1col_sed5.H file2=tmp_hess_11_sed5.H file3=${D}/tri_hess_inv_2col_sed5.H file4=tmp_hess_12_sed5.H file5=${D}/tri_hess_inv_3col_sed5.H file6=tmp_hess_13_sed5.H exp='file1*file2+file3*file4+file5*file6' > test_inv_col_1_sed5.H
	#Second column of I
	Pad n3=3 extend=1 <${D}/hess_12_sed5.H > tmp_hess_12_sed5.H
	Pad n3=3 extend=1 <${D}/hess_22_sed5.H > tmp_hess_22_sed5.H
	Pad n3=3 extend=1 <${D}/hess_23_sed5.H > tmp_hess_23_sed5.H
	Math file1=${D}/tri_hess_inv_1col_sed5.H file2=tmp_hess_12_sed5.H file3=${D}/tri_hess_inv_2col_sed5.H file4=tmp_hess_22_sed5.H file5=${D}/tri_hess_inv_3col_sed5.H file6=tmp_hess_23_sed5.H exp='file1*file2+file3*file4+file5*file6' > test_inv_col_2_sed5.H
	#Third column of I
	Pad n3=3 extend=1 <${D}/hess_13_sed5.H > tmp_hess_13_sed5.H
	Pad n3=3 extend=1 <${D}/hess_23_sed5.H > tmp_hess_23_sed5.H
	Pad n3=3 extend=1 <${D}/hess_33_sed5.H > tmp_hess_33_sed5.H
	Math file1=${D}/tri_hess_inv_1col_sed5.H file2=tmp_hess_13_sed5.H file3=${D}/tri_hess_inv_2col_sed5.H file4=tmp_hess_23_sed5.H file5=${D}/tri_hess_inv_3col_sed5.H file6=tmp_hess_33_sed5.H exp='file1*file2+file3*file4+file5*file6' > test_inv_col_3_sed5.H
	Rm tmp_hess_*_sed5.H



test_inverse_tridiag_hessian_sed5.H: test_inv_col_1_sed5.H test_inv_col_2_sed5.H test_inv_col_3_sed5.H
	Reshape < test_inv_col_1_sed5.H reshape=1,3 > tmp_hess1_sed5.H
	Reshape < test_inv_col_2_sed5.H reshape=1,3 > tmp_hess2_sed5.H
	Reshape < test_inv_col_3_sed5.H reshape=1,3 > tmp_hess3_sed5.H
	Cat tmp_hess1_sed5.H tmp_hess2_sed5.H tmp_hess3_sed5.H axis=1 >$@
	Rm tmp_hess*_sed5.H test_inv_col_1_sed5.H test_inv_col_2_sed5.H test_inv_col_3_sed5.H
	
	
	
