############################################################################################
#Makekefile for pbs jobs for sediment model
############################################################################################
SCRIPTDIR = ${PROPAG}scripts

ifeq ($(HOSTNAME), cees-rcf.stanford.edu)
SEPQ = sep
PBS_TMPL = ${SCRIPTDIR}/pbs_tmpl_rcf.sh
else ifeq ($(HOSTNAME), cees-mazama)
SEPQ = sep
PBS_TMPL = ${SCRIPTDIR}/pbs_tmpl_mazama.sh
else  # old cees-cluster
PBS_TMPL = ${SCRIPTDIR}/pbs_tmpl.sh
SEPQ = Q35
endif

ifeq ($(HOSTNAME), cees-cluster.stanford.edu)
sortbin=~/Setup/sort
else
sortbin=sort
endif

total_jobs=24,200
queue_cap=100,100


clean_pbs:
	rm -rf wrk/*

#For padding the models
${D}/%_pad.H: ${D}/%.H
	Pad <$< beg1=120 end1=120 beg2=120 end2=120 extend=1 >$@


${D}/%.sed.H: ${DNR}/%.sed.H
	Cp $< $@

	
#Initial model and inversion mask
${D}/%.sed.smooth.H: ${D}/%.sed.H
	#Removing water layer
	Window3d < $< n1=50 > water_junk.H
	Window3d < $< f1=50 | Smooth rect1=10 rect2=10 repeat=5 > sed_junk.H
	Cat3d water_junk.H sed_junk.H axis=1 >$@
	Rm sed_junk.H water_junk.H	
initial_model_sed.H: ${D}/vp.sed.smooth_pad.H ${D}/vs.sed.smooth_pad.H ${D}/rho.sed.smooth_pad.H
	Cat3d $^ axis=3 >$@
	
inv_mask_sed.H: ${D}/vp.sed.H
	Math file1=$< exp="file1*0.0+1.0" > mask_junk.H
	Window3d < mask_junk.H f1=50 | Pad beg1=170 | Smooth rect1=3 > mask_junk1.H 
	Pad < mask_junk1.H end1=120 beg2=120 end2=120 n3=3 extend=1 >$@
	Rm mask_junk.H mask_junk1.H

#GENERATING DATA
#Source wavelet	
${D}/wavelet_elastic_sed.H: ${B}/Ricker.x
	${B}/Ricker.x n1=1800 d1=0.004 freq=15. shift=0.15 | Pad extend=1 end2=1 | Pad end2=3 > $@ 
	
${P}/rec_unif_sed.p: ${D}/rho.sed.H ${B}/Geo_uniform.x
	${B}/Geo_uniform.x <$< depth=0. file=$@ >/dev/null
	
${P}/rec_unif_sed_obn.p: ${D}/rho.sed.H ${B}/Geo_uniform.x
	${B}/Geo_uniform.x <$< depth=480. file=$@ >/dev/null

nshots_sed=$(shell awk 'NR==2' Par/source_unif_sed.p | sed -e 's/^[ \t]*//')
${P}/source_unif_sed.p: ${D}/rho.sed.H ${B}/Geo_uniform.x
	${B}/Geo_uniform.x <$< depth=0. dx=100. file=$@ >/dev/null

#Converting velocities into elastic parameters
${D}/lame.sed.H ${D}/mu.sed.H: ${D}/vp.sed.H ${D}/vs.sed.H ${D}/rho.sed.H
	Math file1=${D}/vp.sed.H file2=${D}/vs.sed.H file3=${D}/rho.sed.H exp="file3*(file1*file1-2.0*file2*file2)/1e9" > ${D}/lame.sed.H
	Math file1=${D}/vs.sed.H file2=${D}/rho.sed.H exp="file2*file1*file1/1e9" > ${D}/mu.sed.H

######################################	
#Forward propagation
sed_data_bte.H: solver_prog ${B}/Propagator_elastic_fw.x ${D}/lame.sed.H ${D}/mu.sed.H ${D}/rho.sed.H ${D}/wavelet_elastic_sed.H ${P}/rec_unif_sed.p ${P}/source_unif_sed.p
	mkdir -p wrk
	cp ${D}/lame.sed.H ${D}/mu.sed.H ${D}/rho.sed.H ${D}/wavelet_elastic_sed.H wrk
	${B}/bte_elastic_fw.py pbs_template=${PBS_TMPL} queues=${SEPQ},default queues_cap=${queue_cap} path_out=wrk prefix=sed_data_bte nshots=${nshots_sed} nshots_perjob=1 lambda=lame.sed.H mu=mu.sed.H rho=rho.sed.H par_rec=../${P}/rec_unif_sed.p par_sou=../${P}/source_unif_sed.p wavelet=wavelet_elastic_sed.H dt=8.0000004E-04 total_jobs_cap=${total_jobs}
	Cat3d `ls wrk/sed_data_bte-*.H  | sort -V` axis=4 > $@
	
#Creating pressure data and particle velocities
pressure_vels_sed.H: sed_data_bte.H
	Window3d < $< n2=2 | Stack3d | Transp reshape=1,3,4 plane=23 | Add scale=0.5 > pressure_junk.H
	Window3d < $< n2=2 f2=3 > vels_junk.H
	Cat3d pressure_junk.H vels_junk.H axis=2 >$@
	Rm pressure_junk.H vels_junk.H
	
###########################################################################
#MODEL-INVERSION USING GENERIC SOLVERs
###########################################################################
#Bandpassing wavelet
${D}/wavelet_elastic_sed_bp%.H: ${D}/wavelet_elastic_sed.H	
	Bandpass < $< phase=1 fhi=$* | Bandpass phase=1 fhi=$* | Bandpass phase=1 fhi=$* | Bandpass phase=1 fhi=$* >$@

#Bandpassing data
pressure_vels_sed_bp%.H: pressure_vels_sed.H	
	Bandpass < $< phase=1 fhi=$* | Bandpass phase=1 fhi=$* | Bandpass phase=1 fhi=$* | Bandpass phase=1 fhi=$* >$@

#Better to launch screen and then detach the session to run inversion	
sed_inversion_bp5.H sed_inversion_bp5/inverted_sed_inversion_bp5.H: solver_prog initial_model_sed.H pressure_vels_sed_bp5.H ${D}/wavelet_elastic_sed_bp5.H ${P}/source_unif_sed.p ${P}/rec_unif_sed.p ${B}/Propagator_elastic_fw.x ${B}/Bornfast_elastic_fw.x ${B}/Bornfast_elastic_adj.x inv_mask_sed.H
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp5.H wrk
	${B}/nl_solver.py fwd_nl_cmd_file=${P}/fwd_cmd_multicomp_sed_nl.txt fwd_cmd_file=${P}/fwd_cmd_multicomp_sed_born.txt adj_cmd_file=${P}/adj_cmd_multicomp_sed_born.txt data=pressure_vels_sed_bp5.H init_model=initial_model_sed.H inv_model=inverted_$@ suffix=_sed_inversion_bp5 iteration_movies=obj,model,residual,gradient wrk_dir=sed_inversion_bp5 niter=40 &

#5Hz band with preconditioner with diagonal hessian	
sed_inversion_bp5_diag.H sed_inversion_bp5_diag/inverted_sed_inversion_bp5_diag.H: solver_prog initial_model_sed.H pressure_vels_sed_bp5.H ${D}/wavelet_elastic_sed_bp5.H ${P}/source_unif_sed.p ${P}/rec_unif_sed.p ${B}/Propagator_elastic_fw.x ${B}/Bornfast_elastic_fw.x ${B}/Bornfast_elastic_adj.x inv_mask_sed.H ${D}/norm_diag_hess_inv_sed5.H
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp5.H wrk
	${B}/nl_solver.py fwd_nl_cmd_file=${P}/fwd_cmd_multicomp_sed_nl_diag.txt fwd_cmd_file=${P}/fwd_cmd_multicomp_sed_born_diag.txt adj_cmd_file=${P}/adj_cmd_multicomp_sed_born_diag.txt data=pressure_vels_sed_bp5.H init_model=initial_model_sed.H inv_model=inverted_$@ suffix=_sed_inversion_bp5_diag iteration_movies=obj,model,residual,gradient wrk_dir=sed_inversion_bp5_diag niter=40 &


#Other frequency bands
#Diagonal preconditioner
sed_inversion_bp%_diag.H sed_inversion_bp%_diag/inverted_sed_inversion_bp%_diag.H: solver_prog pressure_vels_sed_bp%.H ${D}/wavelet_elastic_sed_bp%.H ${P}/source_unif_sed.p ${P}/rec_unif_sed.p ${B}/Propagator_elastic_fw.x ${B}/Bornfast_elastic_fw.x ${B}/Bornfast_elastic_adj.x inv_mask_sed.H ${D}/norm_diag_hess_inv_sed%.H
	make sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp$*.H wrk
	sed 's/wavelet_elastic_sed_bp5/wavelet_elastic_sed_bp$*/' < ${P}/fwd_cmd_multicomp_sed_nl_diag.txt > ${P}/tmp_sed_nl_diag.txt
	sed 's/wavelet_elastic_sed_bp5/wavelet_elastic_sed_bp$*/' < ${P}/fwd_cmd_multicomp_sed_born_diag.txt > ${P}/tmp_sed_born_fwd_diag.txt
	sed 's/wavelet_elastic_sed_bp5/wavelet_elastic_sed_bp$*/' < ${P}/adj_cmd_multicomp_sed_born_diag.txt | sed 's/diag_hess_inv_sed5.H/diag_hess_inv_sed$*.H/' > ${P}/tmp_sed_born_adj_diag.txt
	${B}/nl_solver.py fwd_nl_cmd_file=${P}/tmp_sed_nl_diag.txt fwd_cmd_file=${P}/tmp_sed_born_fwd_diag.txt adj_cmd_file=${P}/tmp_sed_born_adj_diag.txt data=pressure_vels_sed_bp$*.H init_model=sed_inversion_bp$(shell echo $*-5| bc)_diag/inverted_sed_inversion_bp$(shell echo $*-5| bc)_diag.H inv_model=inverted_$@ suffix=_sed_inversion_bp$*_diag iteration_movies=obj,model,residual,gradient wrk_dir=sed_inversion_bp$*_diag niter=40 &


#No preconditioner
sed_inversion_bp%.H sed_inversion_bp%/inverted_sed_inversion_bp%.H: solver_prog pressure_vels_sed_bp%.H ${D}/wavelet_elastic_sed_bp%.H ${P}/source_unif_sed.p ${P}/rec_unif_sed.p ${B}/Propagator_elastic_fw.x ${B}/Bornfast_elastic_fw.x ${B}/Bornfast_elastic_adj.x inv_mask_sed.H
	make sed_inversion_bp$(shell echo $*-5| bc)/inverted_sed_inversion_bp$(shell echo $*-5| bc).H
	mkdir -p wrk
	cp ${D}/wavelet_elastic_sed_bp$*.H wrk
	sed 's/wavelet_elastic_sed_bp5/wavelet_elastic_sed_bp$*/' < ${P}/fwd_cmd_multicomp_sed_nl.txt > ${P}/tmp_sed_nl.txt
	sed 's/wavelet_elastic_sed_bp5/wavelet_elastic_sed_bp$*/' < ${P}/fwd_cmd_multicomp_sed_born.txt > ${P}/tmp_sed_born_fwd.txt
	sed 's/wavelet_elastic_sed_bp5/wavelet_elastic_sed_bp$*/' < ${P}/adj_cmd_multicomp_sed_born.txt > ${P}/tmp_sed_born_adj.txt
	${B}/nl_solver.py fwd_nl_cmd_file=${P}/tmp_sed_nl.txt fwd_cmd_file=${P}/tmp_sed_born_fwd.txt adj_cmd_file=${P}/tmp_sed_born_adj.txt data=pressure_vels_sed_bp$*.H init_model=sed_inversion_bp$(shell echo $*-5| bc)/inverted_sed_inversion_bp$(shell echo $*-5| bc).H inv_model=inverted_$@ suffix=_sed_inversion_bp$* iteration_movies=obj,model,residual,gradient wrk_dir=sed_inversion_bp$* niter=40 &
	#rm -f ${P}/tmp_sed_nl.txt ${P}/tmp_sed_born_fwd.txt ${P}/tmp_sed_born_adj.txt
