#!/usr/bin/python
import copy
import os
import sepbase
import tempfile
import unittest
import re
import math

from batch_task_executor import *
import pbs_util


class ElasticBatchTaskComposer(BatchTaskComposer):
	def __init__(self, param_reader, njobs, nfiles_perjob):
		BatchTaskComposer.__init__(self)
		self.njobs = njobs
		self.nfiles_perjob = nfiles_perjob
    	# Create all subjobs info
		self.subjobids = range(0, self.njobs)
		self.subjobfns_list = [None]*self.njobs
		path_out = param_reader.path_out
		# Create files for each subjob.
		for i in range(0,self.njobs):
			nfiles = self.nfiles_perjob
			self.subjobfns_list[i] = [None]*nfiles
			for j in range(0,nfiles):
				self.subjobfns_list[i][j] = '%s/%s-%s-%s.H' % (path_out,param_reader.prefix,i,j)
		return

	def GetSubjobScripts(self, subjobid):
		'''Return the scripts content for that subjob.'''
		subjobfns = self.subjobfns_list[subjobid]
		scripts=[]
    	# Build scripts that create the files designated in subjobfns\
		jobname = subjobfns[0]
		shots=[]
		shots=(re.findall('\d+', jobname ))
		shot_num=int(shots[-2])
		args_ela=eq_args_from_cmdline

		nshots_perjob=shots_per_jobs[shot_num]
		#define shot position file
		cmd = 'mkdir -p Par/ \n'
		scripts.append(cmd)
		cmd = 'rm -f Par/%s-%s.p \n' %(param_reader.prefix,shot_num)
		scripts.append(cmd)
		cmd = 'touch Par/%s-%s.p \n' %(param_reader.prefix,shot_num)
		scripts.append(cmd)
		cmd = 'echo %s >> Par/%s-%s.p \n' %(nshots_perjob,param_reader.prefix,shot_num)
		scripts.append(cmd)
		cmd = 'sed -n %s,%sp %s >> Par/%s-%s.p \n' %(first_shot_number[shot_num]+4,first_shot_number[shot_num]+4+nshots_perjob-1,args_ela['par_sou'],param_reader.prefix,shot_num)
		scripts.append(cmd)
		#Running the executable   >deltalambda.H
		cmd = '../bin/Propagator_elastic_fw.x < %s rho=%s mu=%s wavelet=%s sources=Par/%s-%s.p recs=%s >%s verbose=1 dt=%s threads=%s mod_padded=%s \n' %(args_ela['lambda'],args_ela['rho'],args_ela['mu'],args_ela['wavelet'],param_reader.prefix,shot_num,args_ela['par_rec'],jobname,args_ela['dt'],args_ela['threads'],args_ela['mod_padded'])
		scripts.append(cmd)
		return scripts, str(subjobid)

	def GetSubjobsInfo(self):
		'''Should return two lists, subjobids and subjobfns.'''
		return copy.deepcopy(self.subjobids), copy.deepcopy(self.subjobfns_list)


if __name__ == '__main__':
	if(len(sys.argv) == 1):
		print "NAME \n"
		print "	BatchTaskExecutor_foward_elastic - Batch task executor to run multiple shots on cluster \n"
		print "SYNOPSIS"
		print "	batch_task_executor_elastic_fw.py pbs_template=tmpl.sh queues=default,sep queues_cap=20,20 path_out= prefix= nshots= nshots_perjob=1 lambda=lame.H mu=mu.H rho=rho.H par_rec=./Par/rec_unif.p par_sou=./Par/source_unif.p wavelet=wavelet_elastic.H dt=0.0 total_jobs_cap=100,100\n"
		print "DESCRIPTION"
		print "	Run multiple shots on any CEES cluster"
		print "INPUT PARAMETERS"
		print "	pbs_template= - char"
		print "		[no default]: File containing pbs script template (i.e., file in which commands are added)\n"
		print "	queues= - char"
		print "		[no default]: Queues to use during job submissions\n"
		print "	queues_cap= - char"
		print "		[no default]: Maximum jobs allowed in Q status\n"
		print "	path_out= - char"
		print "		[no default]: Path in which all files are going to be written\n"
		print "	prefix= - char"
		print "		[no default]: Prefix to identify job and file names\n"
		print "	nshots= - int"
		print "		[no default]: Total number of shots to run\n"
		print "	nshots_perjob= - int"
		print "		[1]: Number of shots per job \n"
		print "	lambda= - char"
		print "		[no default]: Lame's parameter model file\n"
		print "	mu= - char"
		print "		[no default]: Shear modulus model file\n"
		print "	rho= - char"
		print "		[no default]: Density model file\n"
		print "	mod_padded= - int"
		print "		[0]:  Model parameters were padded (120 elements), useful for FWI\n"
		print "	par_rec= - char"
		print "		[no default]: Parameter file containing receiver positions\n"
		print "	par_sou= - char"
		print "		[no default]: Parameter file containing source positions\n"
		print "	wavelet= - char"
		print "		[no default]: Elastic wavelet file\n"
		print "	dt= - float"
		print "		[0.0]: Fine dt to use during propagation\n"
		print "	total_jobs_cap= - int"
		print "		[100]: Total number of jobs per queue\n"
		print "	threads= - int"
		print "		[24]:  Number of threads to use during computation\n"
		print ""
	else:
  		print "Run BatchTaskExecutor_foward_elastic with params:", sys.argv
  		# Check basic environment variable setup.
		assert 'SEP' in os.environ, '!Environment var SEP is not set yet, should have your seplib enviroment set up first! Check paths like /opt/SEP/'
		#Parsing command line
		eq_args_from_cmdline,args = sepbase.parse_args(sys.argv)
		#Parsing command line for PBS related parameters
		param_reader = pbs_util.JobParamReader(eq_args_from_cmdline)
		
		#Parameters for running the jobs
		nshots=int(eq_args_from_cmdline['nshots'])
		if eq_args_from_cmdline.has_key('nshots_perjob'):
			nshots_perjob=int(eq_args_from_cmdline['nshots_perjob'])
		else:
			nshots_perjob = 1
		
		if(not eq_args_from_cmdline.has_key('dt')):
			eq_args_from_cmdline['dt']=0.0
		if(not eq_args_from_cmdline.has_key('threads')):
			eq_args_from_cmdline['threads']=24
		if(not eq_args_from_cmdline.has_key('mod_padded')):
			eq_args_from_cmdline['mod_padded']=0
		
		first_shot_number=range(0,nshots,nshots_perjob)
		njobs=len(first_shot_number)
		shots_per_jobs=[nshots_perjob]*(njobs-1)
		shots_per_jobs.append(nshots-first_shot_number[-1])
		
		#Creating job obj to run
		nfiles_perjob=1
		elastic_btc = ElasticBatchTaskComposer(param_reader,njobs,nfiles_perjob)
		
		#Creating executor and launching jobs
		bte = BatchTaskExecutor(param_reader)
		prefix = param_reader.prefix
		bte.LaunchBatchTask(prefix, elastic_btc)
		print 'Run Successful!'
  
